//-----------------------------------------------------------------------------
//  TimpyH_MSD_MMC.c
//-----------------------------------------------------------------------------
//
//  Copyright 2006, 2007 Chiaki Nakajima
//  Original copyright 2006 Silicon Laboratories, Inc.
//
//  Release 1.1
//      E\[X̐
//      EgpvZbTF340ɌC"__F340_VER__"`QƂ폜
//      Et@C"F34x_""TimpyH_"ɕύX
//
//  Release 1.0
//      ESiLabsUSB-MSD-RD(Release 1.1)ɁCTimpy Rev5.0pɑSʓIɕύX
//      ESDCCֈڐA
//      Esvȋ@\폜
//
//-----------------------------------------------------------------------------
//
//  Copyright 2006 Silicon Laboratories, Inc.
//  http://www.silabs.com
//
//  Release 1.1
//    -All changes by PKC
//    -09 JUN 2006
//    -Replaced SFR definitions file "c8051f320.h" with "c8051f340.h"
//    -Corrected "SPIDAT" to "SPI0DAT" in function Write_Read_Spi_Byte
//
//  Release 1.0
//    -Initial Release
//
//-----------------------------------------------------------------------------
//  Includes
//-----------------------------------------------------------------------------

#include "c8051f340.h"
#include "TimpyH_Common_Defs.h"
#include "TimpyH_MSD_MMC.h"

// Constants that define available card sizes, 8MB through 128MB                                       
#define PS_8MB          8388608L
#define PS_16MB         16777216L
#define PS_32MB         33554432L
#define PS_64MB         67108864L
#define PS_128MB        134217728L

#define ERROR_CODE      0xFFFF
#define BUFFER_SIZE     16

// Erase group size = 16 MMC FLASH sectors
#define PHYSICAL_GROUP_SIZE     (PHYSICAL_BLOCK_SIZE * 16)

// Command table value definitions
// Used in the MMC_Command_Exec function to 
// decode and execute MMC command requests
#define EMPTY           0
#define YES             1
#define NO              0
#define CMD             0
#define RD              1
#define WR              2
#define R1              0
#define R1b             1
#define R2              2
#define R3              3

// Start and stop data tokens for single and multiple
// block MMC data operations
#define START_SBR       0xFE
#define START_MBR       0xFE
#define START_SBW       0xFE
#define START_MBW       0xFC
#define STOP_MBW        0xFD

// Mask for data response Token after an MMC write
#define DATA_RESP_MASK  0x11

// Mask for busy Token in R1b response
#define BUSY_BIT        0x80

// Command Table Index Constants:
// Definitions for each table entry in the command table.
// These allow the MMC_Command_Exec function to be called with a
// meaningful parameter rather than a number.
#define GO_IDLE_STATE            0
#define SEND_OP_COND             1
#define SEND_CSD                 2
#define SEND_CID                 3
#define STOP_TRANSMISSION        4
#define SEND_STATUS              5
#define SET_BLOCKLEN             6
#define READ_SINGLE_BLOCK        7
#define READ_MULTIPLE_BLOCK      8
#define WRITE_BLOCK              9
#define WRITE_MULTIPLE_BLOCK    10
#define PROGRAM_CSD             11
#define SET_WRITE_PROT          12
#define CLR_WRITE_PROT          13
#define SEND_WRITE_PROT         14
#define TAG_SECTOR_START        15
#define TAG_SECTOR_END          16
#define UNTAG_SECTOR            17
#define TAG_ERASE_GROUP_START   18
#define TAG_ERASE_GROUP_END     19
#define UNTAG_ERASE_GROUP       20
#define ERASE                   21
#define LOCK_UNLOCK             22
#define READ_OCR                23
#define CRC_ON_OFF              24

//-----------------------------------------------------------------------------
//  UNIONs, STRUCTUREs, and ENUMs
//-----------------------------------------------------------------------------

// This structure defines entries into the command table;
typedef struct {
    BYTE command_byte;          // OpCode;
    BYTE arg_required;          // Indicates argument requirement;
    BYTE CRC;                   // Holds CRC for command if necessary;
    BYTE trans_type;            // Indicates command transfer type;
    BYTE response;              // Indicates expected response;
    BYTE var_length;            // Indicates varialble length transfer;
} COMMAND;

// Command table for MMC.  This table contains all commands available in SPI
// mode;  Format of command entries is described above in command structure
// definition;
COMMAND code command_list[25] = {
    { 0, NO , 0x95, CMD, R1 , NO },     // CMD0;  GO_IDLE_STATE: reset card;
    { 1, NO , 0xFF, CMD, R1 , NO },     // CMD1;  SEND_OP_COND: initialize card;
    { 9, NO , 0xFF, RD , R1 , NO },     // CMD9;  SEND_CSD: get card specific data;
    {10, NO , 0xFF, RD , R1 , NO },     // CMD10; SEND_CID: get card identifier;
    {12, NO , 0xFF, CMD, R1 , NO },     // CMD12; STOP_TRANSMISSION: end read;
    {13, NO , 0xFF, CMD, R2 , NO },     // CMD13; SEND_STATUS: read card status;
    {16, YES, 0xFF, CMD, R1 , NO },     // CMD16; SET_BLOCKLEN: set block size;
    {17, YES, 0xFF, RD , R1 , NO },     // CMD17; READ_SINGLE_BLOCK: read 1 block;
    {18, YES, 0xFF, RD , R1 , YES},     // CMD18; READ_MULTIPLE_BLOCK: read > 1;
    {24, YES, 0xFF, WR , R1 , NO },     // CMD24; WRITE_BLOCK: write 1 block;
    {25, YES, 0xFF, WR , R1 , YES},     // CMD25; WRITE_MULTIPLE_BLOCK: write > 1;
    {27, NO , 0xFF, CMD, R1 , NO },     // CMD27; PROGRAM_CSD: program CSD;
    {28, YES, 0xFF, CMD, R1b, NO },     // CMD28; SET_WRITE_PROT: set wp for group;
    {29, YES, 0xFF, CMD, R1b, NO },     // CMD29; CLR_WRITE_PROT: clear group wp;
    {30, YES, 0xFF, CMD, R1 , NO },     // CMD30; SEND_WRITE_PROT: check wp status;
    {32, YES, 0xFF, CMD, R1 , NO },     // CMD32; TAG_SECTOR_START: tag 1st erase;
    {33, YES, 0xFF, CMD, R1 , NO },     // CMD33; TAG_SECTOR_END: tag end(single);
    {34, YES, 0xFF, CMD, R1 , NO },     // CMD34; UNTAG_SECTOR: deselect for erase;
    {35, YES, 0xFF, CMD, R1 , NO },     // CMD35; TAG_ERASE_GROUP_START;
    {36, YES, 0xFF, CMD, R1 , NO },     // CMD36; TAG_ERASE_GROUP_END;
    {37, YES, 0xFF, CMD, R1 , NO },     // CMD37; UNTAG_ERASE_GROUP;
    {38, YES, 0xFF, CMD, R1b, NO },     // CMD38; ERASE: erase all tagged sectors;
    {42, YES, 0xFF, CMD, R1b, NO },     // CMD42; LOCK_UNLOCK;
    {58, NO , 0xFF, CMD, R3 , NO },     // CMD58; READ_OCR: read OCR register;
    {59, YES, 0xFF, CMD, R1 , NO }      // CMD59; CRC_ON_OFF: toggles CRC checking;
};

//-----------------------------------------------------------------------------
//  Global VARIABLES
//-----------------------------------------------------------------------------

// Removed these. It doesn't work correctly on every MMC card, and we need
// all the resources we can get.
ULONG __xdata PHYSICAL_SIZE;            // MMC size variable;  Set during initialization;
ULONG __xdata PHYSICAL_BLOCKS;          // MMC block number;  Computed during initialization;

bit Is_Initialized;
char __xdata LOCAL_BLOCK[BUFFER_SIZE]; 
UINT __data current_blklen = 512;
static BYTE Write_Read_Spi_Byte(BYTE byte);

extern BYTE __xdata Scratch[PHYSICAL_BLOCK_SIZE];

void Wait_ms(UINT count);
void Wait_us(UINT count);               // Wait_nsA萸x̍Wait_usɕύX

//extern READ_BYTES(BYTE *pchar, UINT len);
//extern WRITE_BYTES(BYTE *pchar, UINT len);
extern void Read_Bytes(BYTE *pchar);
extern void Write_Bytes(BYTE *pchar);
extern void Wait_ms_asm(UINT count);    // AZuɂ
extern void Wait_us_asm(UINT count);    // AZuɂ

#define BACK_FROM_ERROR \
{\
    SPI0DAT = 0xFF; \
    while (!SPIF) {} \
    SPIF = 0; \
    pSdCs = 1; \
    SPI0DAT = 0xFF;\
    while (!SPIF) {} \
    SPIF = 0;  \
    return ERROR_CODE; \
}

//----------------------------------------------------------------------------
//  SPI0_Init
//----------------------------------------------------------------------------
//
//  Configure SPI0 for 8-bit, 2MHz SCK, Master mode, polled operation, data
//  sampled on 1st SCK rising edge.
//
//  Parameters   :
//  Return Value :
//----------------------------------------------------------------------------

void SPI_Init(void)
{
    Is_Initialized = 0;
    SPI0CFG = 0x70;                             // data sampled on rising edge, clk active low, 8-bit data words, master mode;
    SPI0CN = 0x01;                              // 3-wire mode; SPI enabled; flags cleared
    SPI0CKR = 119;
    pSdCs = 1;
}

//-----------------------------------------------------------------------------
//  MMC_Command_Exec
//-----------------------------------------------------------------------------
//
//  This function generates the necessary SPI traffic for all MMC SPI commands.
//  The three parameters are described below:
// 
//  Cmd:      This parameter is used to index into the command table and read
//            the desired command.  The Command Table Index Constants allow the
//            caller to use a meaningful constant name in the Cmd parameter 
//            instead of a simple index number.  For example, instead of calling 
//            MMC_Command_Exec (0, argument, pchar) to send the MMC into idle 
//            state, the user can call 
//            MMC_Command_Exec (GO_IDLE_STATE, argument, pchar);
//
//  argument: This parameter is used for MMC commands that require an argument.
//            MMC arguments are 32-bits long and can be values such as an
//            an address, a block length setting, or register settings for the
//            MMC.
//
//  pchar:    This parameter is a pointer to the local data location for MMC 
//            data operations.  When a read or write occurs, data will be stored
//            or retrieved from the location pointed to by pchar.
//
//  The MMC_Command_Exec function indexes the command table using the Cmd 
//  parameter. It reads the command table entry into memory and uses information
//  from that entry to determine how to proceed.  Returns the 16-bit card 
//  response value;
//

UINT MMC_Command_Exec (BYTE cmd_loc, ULONG argument, BYTE *pchar)
{
    /*xdata*/ BYTE loopguard;
    /*idata*/ COMMAND current_command;          // Local space for the command table entry;
    /*idata*/ ULONG_BA long_arg;                // Union variable for easy byte transfers of the argument;
    ULONG /*xdata*/ old_blklen = 512;           // Temp variable to preserve data block length during temporary changes;
    /*idata*/ UINT counter = 0;                 // Byte counter for multi-byte fields;
    /*idata*/ UINT_BA card_response;            // Variable for storing card response;
    /*idata*/ BYTE data_resp;                   // Variable for storing data response;
    /*idata*/ BYTE dummy_CRC;                   // Dummy variable for storing CRC field;
    BYTE *plast;
   
                                                // Retrieve desired command table entry from code space;
    current_command.command_byte = command_list[cmd_loc].command_byte;
    current_command.arg_required = command_list[cmd_loc].arg_required;
    current_command.CRC          = command_list[cmd_loc].CRC;
    current_command.trans_type   = command_list[cmd_loc].trans_type;
    current_command.response     = command_list[cmd_loc].response;
    current_command.var_length   = command_list[cmd_loc].var_length;
//  current_command = command_list[cmd_loc];    // Retrieve desired command table entry from code space;

    card_response.i = 0;
    Write_Read_Spi_Byte(0xFF);                  // Send buffer SPI clocks to ensure no MMC operations are pending;
    pSdCs = 0;                                  // Select MMC by pulling CS low;
    Write_Read_Spi_Byte(0xFF);                  // Send another byte of SPI clocks;
    Write_Read_Spi_Byte(current_command.command_byte | 0x40);
    long_arg.l = argument;                      // Make argument byte addressable;
                                                // If current command changes block length, update block length variable
                                                // to keep track;
                                                // Command byte = 16 means that a set block length command is taking place
                                                // and block length variable must be set;
    if (current_command.command_byte == 16) {
        current_blklen = argument;
    }
                                                // Command byte = 9 or 10 means that a 16-byte register value is being read
                                                // from the card, block length must be set to 16 bytes, and restored at the
                                                // end of the transfer;
    if ((current_command.command_byte == 9)||
       (current_command.command_byte == 10)) {
        old_blklen = current_blklen;            // Command is a GET_CSD or GET_CID,
        current_blklen = 16;                    // set block length to 16-bytes;
    }

                                                // Wait for initial SPI transfer to end;
                                                // If an argument is required, transmit one, otherwise transmit 4 bytes of 0x00;
    plast = &pchar[current_blklen];
    if (current_command.arg_required == YES) {
        counter = 4;
        do {
            counter--;
            Write_Read_Spi_Byte(long_arg.b[counter]);
        } while (counter);
    } else {
        counter = 0;
        while (counter <= 3) {
            Write_Read_Spi_Byte(0x00);
            counter++;
        }
    }
    Write_Read_Spi_Byte(current_command.CRC);   // Transmit CRC byte;  In all cases except CMD0, this will be a dummy character;

                                                // The command table entry will indicate what type of response to expect for
                                                // a given command;  The following conditional handles the MMC response;
    if (current_command.response == R1) {       // Read the R1 response from the card;
        loopguard = 0;
        do {
            card_response.b[0] = Write_Read_Spi_Byte(0xFF);     // Write dummy value to SPI so that the response byte will be shifted in;
            if (!++loopguard) break;
            if (card_response.b[0] & BUSY_BIT) { 
//                Wait_ns(700);
                Wait_us(1);
            }
        } while ((card_response.b[0] & BUSY_BIT));

        if(!loopguard) { 
            return card_response.b[0];
        }

    } else if (current_command.response == R1b) {               // Read the R1b response;
        loopguard = 0;
        do {
            card_response.b[0] = Write_Read_Spi_Byte(0xFF);     // Start SPI transfer;
            if (!++loopguard) break;
        } while ((card_response.b[0] & BUSY_BIT));

        while (Write_Read_Spi_Byte(0xff) == 0x00) {             // Wait for busy signal to end;
            /* Do nothing */
        }

    } else if (current_command.response == R2) {                // Read R2 response
        loopguard = 0;
        do {
            card_response.b[0] = Write_Read_Spi_Byte(0xFF);     // Start SPI transfer;
            if (!++loopguard) break;
        } while ((card_response.b[0] & BUSY_BIT));

        if (!loopguard) { 
            BACK_FROM_ERROR; 
        }

        card_response.b[1] = Write_Read_Spi_Byte(0xFF);
        
    } else {                                                    // Read R3 response;
        loopguard = 0;
        do {
            card_response.b[0] = Write_Read_Spi_Byte(0xFF);     // Start SPI transfer;
            if (!++loopguard) break;
        } while ((card_response.b[0] & BUSY_BIT));

        if (!loopguard) { 
            BACK_FROM_ERROR; 
        }

        counter = 0;
        while (counter <= 3) {                                  // Read next three bytes and store them in local memory;
                                                                // These bytes make up the Operating Conditions Register
            counter++;
            *pchar++ = Write_Read_Spi_Byte(0xFF);               // (OCR);
        }
    }
    
    switch (current_command.trans_type) {                       // This conditional handles all data operations;  
                                                                // The command entry determines what type, if any, data
                                                                // operations need to occur;
        case RD:                                                // Read data from the MMC;
            loopguard = 0;
            while ((Write_Read_Spi_Byte(0xFF)) != START_SBR) {  // Wait for a start read Token from the MMC;
//                Wait_ns(700);
                Wait_us(1);
                if (!++loopguard) { 
                    BACK_FROM_ERROR;
                }
            }

            counter = 0;                                        // Reset byte counter;
//          START_SPI_TIMEOUT;
//          READ_BYTES(pchar,current_blklen);
            Read_Bytes(pchar);                                  // Read <current_blklen> bytes;
//          STOP_SPI_TIME_OUT;
            dummy_CRC = Write_Read_Spi_Byte(0xFF);              // After all data is read, read the two CRC bytes;
            dummy_CRC = Write_Read_Spi_Byte(0xFF);
            break;

        case WR:                                                // Write data to the MMC;
            Write_Read_Spi_Byte(0xFF);                          // Start by sending 8 SPI clocks so the MMC can prepare for the write;
            Write_Read_Spi_Byte(START_SBW);
//            Wait_ns(700);
            Wait_us(1);

//          START_SPI_TIMEOUT;
//          WRITE_BYTES(pchar,current_blklen);
            Write_Bytes(pchar);
//          STOP_SPI_TIME_OUT;
            Write_Read_Spi_Byte(0xFF);                          // Write CRC bytes (don't cares);
            Write_Read_Spi_Byte(0xFF);

            loopguard = 0;
            do {                                                // Read Data Response from card;
                data_resp = Write_Read_Spi_Byte(0xFF);
                if (!++loopguard) break;
            } while((data_resp & DATA_RESP_MASK) != 0x01);      // When bit 0 of the MMC response is clear, a valid data response
                                                                // has been received;
        
            if (!loopguard) { 
                BACK_FROM_ERROR; 
            }

            while (Write_Read_Spi_Byte(0xFF) == 0x00) {         // Wait for end of busy signal;
                /* Do nothing */
            }
            
            Write_Read_Spi_Byte(0xFF);                          // Issue 8 SPI clocks so that all card operations can complete;
            break;
            
        default: 
            break;
    }

    Write_Read_Spi_Byte(0xFF);
    pSdCs = 1;                                                  // Deselect memory card;
    Write_Read_Spi_Byte(0xFF);                                  // Send 8 more SPI clocks to ensure the card has finished all 
                                                                // necessary operations;

    if ((current_command.command_byte == 9)
     || (current_command.command_byte == 10)) {
        current_blklen = old_blklen;                            // Restore old block length if needed;
    }
    return card_response.i;
}

//-----------------------------------------------------------------------------
//  MMC_FLASH_Init
//-----------------------------------------------------------------------------
//
//  This function initializes the flash card, configures it to operate in SPI
//  mode, and reads the operating conditions register to ensure that the device
//  has initialized correctly.  It also determines the size of the card by 
//  reading the Card Specific Data Register (CSD).

void MMC_FLASH_Init(void)
{
    /*xdata*/ UINT loopguard;
    /*xdata*/ UINT_BA card_status;              // Stores card status returned from MMC function calls(MMC_Command_Exec);
    /*xdata*/ BYTE counter = 0;                 // SPI byte counter;
    BYTE /*xdata*/ *pchar;                      // Xdata pointer for storing MMC register values;

    UINT c_size, bl_len;
    BYTE c_mult;

    SPI_Init();
    Wait_ms(100);
    pchar = (BYTE __xdata *) LOCAL_BLOCK;

    for (counter = 0; counter < 8; counter++) { // Transmit at least 64 SPI clocks before any bus comm occurs.
        Write_Read_Spi_Byte(0xFF);
    }
    pSdCs = 0;                                  // Select the MMC with the CS pin;
    for (counter = 0; counter < 2; counter++) { // Send 16 more SPI clocks to ensure proper startup;
        Write_Read_Spi_Byte(0xFF);
    }
    Wait_ms(1);
                                                // Send the GO_IDLE_STATE command with CS driven low;
                                                // This causes the MMC to enter SPI mode;
    card_status.i = MMC_Command_Exec(GO_IDLE_STATE, EMPTY, EMPTY);

    loopguard = 0;
    do {                                        // Send the SEND_OP_COND command// until the MMC indicates that it is
        Wait_ms(1);
        card_status.i = MMC_Command_Exec(SEND_OP_COND, EMPTY, EMPTY);
        if (!++loopguard) break;
    } while ((card_status.b[0] & 0x01));

    if (!loopguard) return;

    Write_Read_Spi_Byte(0xFF);                  // Send 8 more SPI clocks to complete the initialization sequence;

    loopguard = 0;
    do {                                        // Read the Operating Conditions Register (OCR);
        card_status.i = MMC_Command_Exec(READ_OCR, EMPTY, pchar);
        if (!++loopguard) break;
    } while (!(*pchar & 0x80));                 // Check the card busy bit of the OCR;

    if (!loopguard) return;

    card_status.i = MMC_Command_Exec(SEND_STATUS, EMPTY, EMPTY);
                                                // Get the Card Specific Data (CSD) register to determine the size of the MMC;
    card_status.i = MMC_Command_Exec(SEND_CSD, EMPTY, pchar);

    if (card_status.i == 0) {
#ifdef F340_M24
    SPI0CKR = 0;                                // ̃ASYł͂OɂĂقƂǑx͕ςȂD1ZN^Cg1.4msecD
#else
    SPI0CKR = 1;
#endif
    Wait_ms(1);
    } else {
        PHYSICAL_BLOCKS = 0;
        PHYSICAL_SIZE = PHYSICAL_BLOCKS * bl_len;
        return;
    }

    card_status.i = MMC_Command_Exec(SET_BLOCKLEN, (ULONG) PHYSICAL_BLOCK_SIZE, EMPTY);

    bl_len = 1 << (pchar[5] & 0x0f) ;
    c_size = ((pchar[6] & 0x03) << 10) | (pchar[7] << 2) | ((pchar[8] & 0xc0) >> 6);
    c_mult = (((pchar[9] & 0x03) << 1) | ((pchar[10] & 0x80) >> 7));
  
                                                // Determine the number of MMC sectors;
    PHYSICAL_BLOCKS = (ULONG) (c_size + 1) * (1 << (c_mult + 2));
    PHYSICAL_SIZE = PHYSICAL_BLOCKS * bl_len;

    loopguard = 0;

    while ((MMC_FLASH_Block_Read(0, Scratch) != 0)) {
        if (!++loopguard) break;
    }

    Is_Initialized = 1;
    pUsbAccessLed = 1;
}

//-----------------------------------------------------------------------------
//  MMC_FLASH_Block_Read
//-----------------------------------------------------------------------------
//
//  If you know beforehand that you'll read an entire 512-byte block, then
//  this function has a smaller ROM footprint than MMC_FLASH_Read.
//
//  Parameters   : address - address of block
//                 pchar - pointer to byte 
//  Return Value : card status
//----------------------------------------------------------------------------

UINT MMC_FLASH_Block_Read(ULONG address, BYTE *pchar) 
{
    /*xdata*/ UINT card_status;                 // Stores MMC status after each MMC command;
    
    address *= PHYSICAL_BLOCK_SIZE;
    pUsbAccessLed = 0;
    card_status = MMC_Command_Exec(READ_SINGLE_BLOCK, address, pchar);
    pUsbAccessLed = 1;

    return card_status;
}

//-----------------------------------------------------------------------------
//  MMC_FLASH_Block_Write
//-----------------------------------------------------------------------------
//
//  If you know beforehand that you'll write an entire 512-byte block, then
//  this function is more RAM-efficient than MMC_FLASH_Write because it 
//  doesn't require a 512-byte Scratch buffer (and it's faster too, it doesn't
//  require a read operation first). And it has a smaller ROM footprint too.
//
//  Parameters   : address - address of block
//                 wdata - pointer to data 
//  Return Value : card status
//----------------------------------------------------------------------------

BYTE MMC_FLASH_Block_Write(ULONG address, BYTE *wdata) 
{
    /*xdata*/ UINT card_status;                 // Stores status returned from MMC;

    address *= PHYSICAL_BLOCK_SIZE;
    pUsbAccessLed = 0;
    card_status = MMC_Command_Exec(WRITE_BLOCK, address, wdata);
    pUsbAccessLed = 1;

    return card_status;
}

//----------------------------------------------------------------------------
//  Write_Read_Spi_Byte
//----------------------------------------------------------------------------
//
//  Function sends one byte to spi and reads ony byte from spi
//  it will be written with SYSCLK as clk
//
//  Parameters   : byte - value to write
//  Return Value : SPI byte
//----------------------------------------------------------------------------

BYTE Write_Read_Spi_Byte(BYTE byte)
{
    BYTE /*xdata*/ ret;

//  SPIF = 0;                                   // !!
    SPI0DAT = byte;
    while (!SPIF) {
        /* Do nothing */
    }
    SPIF = 0;
    ret = SPI0DAT;
    
    return ret;
}

//----------------------------------------------------------------------------
//  Wait_ms
//----------------------------------------------------------------------------
//
//  Delay function with declared wait time in milliseconds
//
//  Parameters   : count - time in ms
//  Return Value :
//----------------------------------------------------------------------------

void Wait_ms(UINT count)
{
    Wait_ms_asm(count);                         // AZuɂ
}

//----------------------------------------------------------------------------
//  Wait_us
//----------------------------------------------------------------------------
//
//  Delay function with declared wait time in microseconds
//
//  Parameters   : count - time in us
//  Return Value :
//----------------------------------------------------------------------------

void Wait_us(UINT count)
{
    Wait_us_asm(count);                         // AZuɂ
}

//----------------------------------------------------------------------------
//  Get_Status_MMC
//----------------------------------------------------------------------------
//
//  Function returns the status of MMC card
//
//  Parameters   : 
//  Return Value : status
//----------------------------------------------------------------------------

int Get_Status_MMC(void)
{
    return MMC_Command_Exec(SEND_STATUS, EMPTY, EMPTY);
}

/* End of TimpyH_MSD_MMC.c */
